package com.persagy.dmp.admin.jms.executor;

import com.persagy.dmp.admin.jms.MessageExecutor;
import com.persagy.dmp.admin.service.trans.TransObjectContext;
import com.persagy.dmp.admin.service.DmpTransObjectService;
import com.persagy.dmp.rwd.model.DmpMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author: yaoll
 * @date: 2020-12-30
 * @verison: 1.0
 */
@Slf4j
@Component
public class TransObjectExecutor implements MessageExecutor {

	@Autowired
	private DmpTransObjectService transObjectService;

	@Override
	public void execute(DmpMessage message) {

		TransObjectContext ctx = prepareContext(message);

		try {
			// 启动
			transObjectService.start(ctx);

			// 加载中台类型定义
			transObjectService.initClass(ctx);

			// 传输项目
			transObjectService.transProject(ctx);

			// 传输建筑
			transObjectService.transBuilding(ctx);

			// 传输楼层
			transObjectService.transFloor(ctx);

			// 传输空间
			transObjectService.transSpace(ctx);

			// 传输系统设备部件
			transObjectService.transEquipment(ctx);

			// 传输竖井
			transObjectService.transShaft(ctx);

			// TODO 传输虚拟对象

			// 传输关系
			transObjectService.transRelation(ctx);

			// 处理失效数据
			transObjectService.transInvalidObject(ctx);


		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			transObjectService.over(ctx);
		}


	}


	private TransObjectContext prepareContext(DmpMessage message) {
		String groupCode = message.getGroupCode();
		String projectId = message.getProjectId();
		String server = message.getStr1();
		String secret = message.getStr2();

		TransObjectContext context = new TransObjectContext();
		context.setAppId("datautils");
		context.setUserId("system");
		context.setGroupCode(groupCode);
		context.setProjectId(projectId);
		context.setServer(server);
		context.setSecret(secret);
		return context;
	}
}
