package com.persagy.dmp.report.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.criteria.JpaUtils;
import com.persagy.common.date.DateUtils;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.report.common.json.JsonMapper;
import com.persagy.dmp.report.entity.*;
import com.persagy.dmp.report.enumeration.DateType;
import com.persagy.dmp.report.feign.AlarmClient;
import com.persagy.dmp.report.feign.RwdClient;
import com.persagy.dmp.report.model.AlarmRecordModel;
import com.persagy.dmp.report.repository.ReportAlarmRecordRepository;
import com.persagy.dmp.report.repository.ReportObjectRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author fengyanjie
 */
@Slf4j
@Service
public class DataExecutorService {
    @Autowired
    private ReportObjectRepository reportObjectRepository;

    @Autowired
    private ReportAlarmRecordRepository reportAlarmRecordRepository;

    @Autowired
    private ReportUtils reportUtils;

    @Autowired
    private RwdClient rwdClient;

    @Autowired
    private AlarmClient alarmClient;

    @Autowired
    private GroupColumnService groupColumnService;


    @Transactional
    public void reportRwdObject(DateType dateType, Date startTime, Date endTime) throws Exception {
        String tableName = "rwd_object";
        String groupCode = DmpParameterStorage.getGroupCode();
        String projectId = DmpParameterStorage.getProjectId();
        List<Object[]> classCodeObjType = groupColumnService.classCodeObjType();
        if (null == classCodeObjType) {
            return;
        }
        List<ObjectNode> objectList = new ArrayList<>();
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("createTime").gte(startTime).lt(endTime);
        criteria.add("valid", 1);
        for (Object[] item : classCodeObjType) {
            criteria.add("classCode", item[0].toString());
            criteria.add("objType", item[1].toString());
            PagedResponse<ObjectNode> objectResponse = rwdClient.queryObject(projectId, groupCode, criteria);
            objectList.addAll(objectResponse.getData());
        }
        //projectId#groupCode#objType#classCode#dateType#year#month#day#hour#season#date
        Map<String, Long> objectMap = objectList.stream().collect(
                Collectors.groupingBy(item -> {
                    StringBuilder sb = new StringBuilder();
                    sb.append(JsonMapper.getString(item, "projectId")).append("#");
                    sb.append(groupCode).append("#");
                    sb.append(JsonMapper.getString(item, "objType")).append("#");
                    sb.append(JsonMapper.getString(item, "classCode")).append("#");
                    sb.append(dateType).append("#");
                    Date createTime = new Date(Long.parseLong(JsonMapper.getString(item, "createTime")));
                    appendDate(sb, dateType, createTime);
                    return sb.toString();
                }, Collectors.counting()));

        List<BaseEntity> data = new ArrayList<>();
        objectMap.entrySet().forEach(item -> {
            String[] keyArray = item.getKey().split("#");
            ReportRwdObject reportRwdObject = new ReportRwdObject();
            reportRwdObject.setProjectId(keyArray[0]);
            reportRwdObject.setGroupCode(keyArray[1]);
            reportRwdObject.setObjType(keyArray[2]);
            reportRwdObject.setClassCode(keyArray[3]);
            reportRwdObject.setDateType(DateType.valueOf(keyArray[4]));
            reportRwdObject.setYear("".equals(keyArray[5]) ? null : keyArray[5]);
            reportRwdObject.setMonth("".equals(keyArray[6]) ? null : keyArray[6]);
            reportRwdObject.setDay("".equals(keyArray[7]) ? null : keyArray[7]);
            reportRwdObject.setHour("".equals(keyArray[8]) ? null : keyArray[8]);
            reportRwdObject.setSeason("".equals(keyArray[9]) ? null : keyArray[9]);
            reportRwdObject.setCreateTime("".equals(keyArray[10]) ? null : keyArray[10]);
            reportRwdObject.setRwdObjectNum(item.getValue());
            data.add(reportRwdObject);
        });
        ReportConfig reportConfig = reportUtils.queryReportConfig(tableName);
        List<Object> tableGroup = JsonMapper.json2List(JsonMapper.toSimpleJson(reportConfig.getTableGroup()));
        List<ReportConfigGroup> groupList = reportUtils.getGroupList(tableGroup, dateType, startTime, endTime);
        List<BaseEntity> resultList = reportUtils.initData(0, groupList, ReportRwdObject.class, new ArrayList<>());
        List<BaseEntity> result = reportUtils.addList(data, resultList, reportConfig.getCalculatedColumns());

        Map<String, String> map = reportUtils.getDate(dateType, startTime, endTime);
        String start = map.get("start");
        String end = map.get("end");
        QReportRwdObject qt = QReportRwdObject.reportRwdObject;
        BooleanExpression ex = qt.dateType.eq(dateType).and(qt.createTime.goe(start)).and(qt.createTime.lt(end));
        Iterable<ReportRwdObject> all = reportObjectRepository.findAll(ex);
        reportObjectRepository.deleteAll(all);

        List<ReportRwdObject> save = new ArrayList<>();
        result.forEach(item -> save.add((ReportRwdObject) item));
        reportObjectRepository.saveAll(save);
    }

    @Transactional
    public void reportAlarmRecord(DateType dateType, Date startTime, Date endTime) throws Exception {
        String tableName = "alarm_record";
        String groupCode = DmpParameterStorage.getGroupCode();
        String projectId = DmpParameterStorage.getProjectId();
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("createTime").gte(startTime).lt(endTime);
        criteria.add("valid", 1);
        PagedResponse<AlarmRecordModel> alarmRecordResponse = alarmClient.query(projectId, groupCode, "system", criteria);

        if (!alarmRecordResponse.success()) {
            return;
        }
        List<AlarmRecordModel> alarmRecordList = alarmRecordResponse.getData();

        //projectId#groupCode#itemCode#category#classCode#level#state#dateType#year#month#day#hour#season#date
        Map<String, Long> alarmRecordMap = alarmRecordList.stream().collect(
                Collectors.groupingBy(item -> {
                    StringBuilder sb = new StringBuilder();
                    sb.append(item.getProjectId()).append("#");
                    sb.append(groupCode).append("#");
                    sb.append(item.getItemCode()).append("#");
                    sb.append(item.getCategory()).append("#");
                    sb.append(item.getClassCode()).append("#");
                    sb.append(item.getLevel()).append("#");
                    sb.append(item.getState()).append("#");
                    sb.append(dateType).append("#");
                    Date createTime = item.getCreateTime();
                    appendDate(sb, dateType, createTime);
                    return sb.toString();
                }, Collectors.counting()));
        List<BaseEntity> data = new ArrayList<>();
        alarmRecordMap.entrySet().forEach(item -> {
            String[] keyArray = item.getKey().split("#");
            ReportAlarmRecord reportAlarmRecord = new ReportAlarmRecord();
            reportAlarmRecord.setProjectId(keyArray[0]);
            reportAlarmRecord.setGroupCode(keyArray[1]);
            reportAlarmRecord.setItemCode(keyArray[2]);
            reportAlarmRecord.setCategory(keyArray[3]);
            reportAlarmRecord.setClassCode(keyArray[4]);
            reportAlarmRecord.setLevel(keyArray[5]);
            reportAlarmRecord.setAlarmState(keyArray[6]);
            reportAlarmRecord.setDateType(DateType.valueOf(keyArray[4]));
            reportAlarmRecord.setYear("".equals(keyArray[5]) ? null : keyArray[5]);
            reportAlarmRecord.setMonth("".equals(keyArray[6]) ? null : keyArray[6]);
            reportAlarmRecord.setDay("".equals(keyArray[7]) ? null : keyArray[7]);
            reportAlarmRecord.setHour("".equals(keyArray[8]) ? null : keyArray[8]);
            reportAlarmRecord.setSeason("".equals(keyArray[9]) ? null : keyArray[9]);
            reportAlarmRecord.setCreateTime("".equals(keyArray[10]) ? null : keyArray[10]);
            reportAlarmRecord.setAlarmRecordNum(item.getValue());
            data.add(reportAlarmRecord);
        });
        ReportConfig reportConfig = reportUtils.queryReportConfig(tableName);
        List<Object> tableGroup = JsonMapper.json2List(JsonMapper.toSimpleJson(reportConfig.getTableGroup()));
        List<ReportConfigGroup> groupList = reportUtils.getGroupList(tableGroup, dateType, startTime, endTime);
        List<BaseEntity> resultList = reportUtils.initData(0, groupList, ReportAlarmRecord.class, new ArrayList<>());
        List<BaseEntity> result = reportUtils.addList(data, resultList, reportConfig.getCalculatedColumns());

        Map<String, String> map = reportUtils.getDate(dateType, startTime, endTime);
        String start = map.get("start");
        String end = map.get("end");
        QReportAlarmRecord qt = QReportAlarmRecord.reportAlarmRecord;
        BooleanExpression ex = qt.dateType.eq(dateType).and(qt.createTime.goe(start)).and(qt.createTime.lt(end));
        Iterable<ReportAlarmRecord> all = reportAlarmRecordRepository.findAll(ex);
        reportAlarmRecordRepository.deleteAll(all);

        List<ReportAlarmRecord> save = new ArrayList<>();
        result.forEach(item -> save.add((ReportAlarmRecord) item));
        reportAlarmRecordRepository.saveAll(save);
    }

    private void appendDate(StringBuilder sb, DateType dateType, Date createTime) {
        String year = DateUtils.format(createTime, DateUtils.PATTERN_yyyy);
        String month = DateUtils.format(createTime, DateUtils.PATTERN_MM);
        String day = DateUtils.format(createTime, "dd");
        String hour = DateUtils.format(createTime, "HH");
        if (DateType.YEAR == dateType) {
            sb.append(year).append("#");//year
            sb.append("#");//month
            sb.append("#");//day
            sb.append("#");//hour
            sb.append("#");//season
            sb.append(year);//date
        }
        if (DateType.MONTH == dateType) {
            sb.append(year).append("#");//year
            sb.append(month).append("#");//month
            sb.append("#");//day
            sb.append("#");//hour
            sb.append("#");//season
            sb.append(year + month).append("#");//date
        }
        if (DateType.DAY == dateType) {
            sb.append(year).append("#");//year
            sb.append(month).append("#");//month
            sb.append(day).append("#");//day
            sb.append("#");//hour
            sb.append("#");//season
            sb.append(year + month + day);//date
        }
        if (DateType.HOUR == dateType) {
            sb.append(year).append("#");//year
            sb.append(month).append("#");//month
            sb.append(day).append("#");//day
            sb.append(hour).append("#");//hour
            sb.append("#");//season
            sb.append(year + month + day + hour);//date
        }
        if (DateType.SEASON == dateType) {
            //TODO
        }
    }
}
