package com.persagy.dmp.report.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.report.common.criteria.CriteriaUtils;
import com.persagy.dmp.report.common.criteria.JacksonCriteria;
import com.persagy.dmp.report.entity.BaseEntity;
import com.persagy.dmp.report.entity.QReportAlarmRecord;
import com.persagy.dmp.report.entity.ReportAlarmRecord;
import com.persagy.dmp.report.enumeration.DateType;
import com.querydsl.core.Tuple;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * @author fengyanjie
 */
@Service
public class ReportAlarmRecordService implements BaseService {

    @Override
    public List<BooleanExpression> parse(ObjectNode criteria) {
        List<BooleanExpression> exps = new LinkedList<>();
        QReportAlarmRecord qt = QReportAlarmRecord.reportAlarmRecord;
        exps.addAll(CriteriaUtils.parse(qt.id, criteria.get("id")));
        exps.addAll(CriteriaUtils.parse(qt.itemCode, criteria.get("itemCode")));
        exps.addAll(CriteriaUtils.parse(qt.projectId, criteria.get("projectId")));
        exps.addAll(CriteriaUtils.parse(qt.groupCode, criteria.get("groupCode")));
        exps.addAll(CriteriaUtils.parse(qt.category, criteria.get("category")));
        exps.addAll(CriteriaUtils.parse(qt.classCode, criteria.get("classCode")));
        exps.addAll(CriteriaUtils.parse(qt.level, criteria.get("level")));
        exps.addAll(CriteriaUtils.parse(qt.alarmState, criteria.get("alarm_state")));
        exps.addAll(CriteriaUtils.parse(qt.createTime, criteria.get("createTime")));
        exps.addAll(CriteriaUtils.parse(qt.alarmRecordNum, criteria.get("alarmRecordNum")));
        exps.addAll(CriteriaUtils.parse(qt.dateType, criteria.get("dateType"), DateType.class));
        return exps;
    }

    @Override
    public List<BaseEntity> getData(PagedResponse<Tuple> tuplePagedResponse, List<JacksonCriteria.Calculated> calculatedColumns) {
        List<BaseEntity> list = new ArrayList<>();
        QReportAlarmRecord qt = QReportAlarmRecord.reportAlarmRecord;
        tuplePagedResponse.getData().forEach(item -> {
            ReportAlarmRecord entity = new ReportAlarmRecord();
            entity.setId(item.get(qt.id));
            entity.setItemCode(item.get(qt.itemCode));
            entity.setProjectId(item.get(qt.projectId));
            entity.setGroupCode(item.get(qt.groupCode));
            entity.setCategory(item.get(qt.category));
            entity.setClassCode(item.get(qt.classCode));
            entity.setLevel(item.get(qt.level));
            entity.setAlarmState(item.get(qt.alarmState));
            entity.setYear(item.get(qt.year));
            entity.setMonth(item.get(qt.month));
            entity.setDay(item.get(qt.day));
            entity.setHour(item.get(qt.hour));
            entity.setSeason(item.get(qt.season));
            entity.setWeek(item.get(qt.week));
            entity.setCreateTime(item.get(qt.createTime));
            entity.setAlarmRecordNum(item.get(qt.alarmRecordNum));
            entity.setDateType(item.get(qt.dateType));
            List<JacksonCriteria.Calculated> calculatedList = new ArrayList<>();
            for (int i = 0; i < calculatedColumns.size(); i++) {
                JacksonCriteria.Calculated calculated = new JacksonCriteria.Calculated();
                if (calculatedColumns.get(i).getColumn().equals("alarmRecordNum")) {
                    calculated.setColumn(calculatedColumns.get(i).getColumn());
                    calculated.setFunction(calculatedColumns.get(i).getFunction());
                    calculated.setValue(item.get(i, Long.class));
                }
                if (calculatedColumns.get(i).getColumn().equals("alarmRecordNum2")) {
                    calculated.setColumn(calculatedColumns.get(i).getColumn());
                    calculated.setFunction(calculatedColumns.get(i).getFunction());
                    calculated.setValue(item.get(i, Long.class));
                }
                calculatedList.add(calculated);
            }
            entity.setCalculatedColumns(calculatedList);
            list.add(entity);
        });
        return list;
    }
}
